/*
 * BMCpp
 * Copyright (C) 2016, 2017 Wilhelm M <guessmail>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef NDEBUG
# define NDEBUG
#endif

#include "mcu/avr8.h"
#include "mcu/ports.h"

using namespace AVR;

using PortB = Port<ATMega328P::PortRegister, B>;
using PortD = Port<ATMega328P::PortRegister, D>;

using engine       = ActiveHigh<Pin<PortB, 1>, Output>;
using lights       = ActiveHigh<Pin<PortB, 2>, Output>;
using startButton  = ActiveLow<Pin<PortD, 1>, Input>;
using brakes       = ActiveLow<Pin<PortD, 2>, Input>;
using gear         = ActiveLow<Pin<PortD, 3>, Input>;

template<typename... C>
struct IO final {
    IO() = delete;
    static void init() {
        (C::init(), ...);
    }
};

int main() {
    IO<engine, lights, startButton, brakes, gear>::init();

    while(true) {
         if (startButton::isActive()) {
             lights::activate();
             if (gear::isActive() && brakes::isActive()) {
                 engine::activate();
             }
         } else {
             engine::inactivate();
             lights::inactivate();
         }
     }
}
